#include <stdio.h>

/* Ausgabe einer nicht-negativen ganzen Zahl in Binaerdarstellung;
   die Zahl der ausgegebenen Zeichen wird zurueckgegeben
*/
unsigned int print_binary(unsigned int n, unsigned int minwidth) {
   unsigned int rn; /* n in bit-maessig umgedrehter Reihenfolge */
   unsigned int printed = 0; /* Anzahl der ausgegebenen Zeichen */
   unsigned int count = 0; /* Anzahl der relevanten Ziffern */

   /* Reihenfolge der Bits umdrehen */
   for (rn = 0; n != 0; n >>= 1) {
      rn = (rn << 1) | (n & 1);
      ++count;
   }
   /* Ausgabe der fuehrenden Nullen, falls notwendig */
   if (minwidth > count) {
      minwidth -= count;
      while (minwidth > 0) {
         putchar('0'); ++printed; --minwidth;
      }
   }
   /* Ausgabe der relevanten Ziffern */
   for (; count > 0; rn >>= 1) {
      if (rn & 1) {
         putchar('1');
      } else {
         putchar('0');
      }
      ++printed; --count;
   }
   return printed;
}

int main() {
   unsigned int i = 17, j = 3;

   printf("     i = "); print_binary(i, 5); puts("");
   printf("     j = "); print_binary(j, 5); puts("");

   /* Bitweises UND, ODER und XOR (= exklusives ODER) */
   printf("   i&j = "); print_binary(i & j, 5);
   printf(",  i|j = "); print_binary(i | j, 5);
   printf(",  i^j = "); print_binary(i ^ j, 5);
   puts("");

   /* Bit-Verschiebungen (nicht zyklisch!) */
   printf("  i>>1 = "); print_binary(i >> 1, 5);
   printf(", i<<1 = "); print_binary(i << 1, 5);
   puts("");

   /* Bit-Verschiebungen bei negativen Zahlen (undefiniert!)
      (bei der Konvertierung von signed zu unsigned garantiert
      C hier das Zweier-Komplement bei negativen Zahlen)
   */
   printf("    -1 = ");
   unsigned int bits = print_binary(-1, 0); puts("");
   printf(" -1>>1 = "); print_binary(-1 >> 1, bits); puts("");
   printf(" -1<<1 = "); print_binary(-1 << 1, bits); puts("");
}
